/*
Testsignal fr DCF77 mit Schaltsekunde
ATTiny85
*/

#include "sbit.h"
#include "main.h"

#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>

#define TIMER_SET   (256 - F_CPU / 256 / 125)
#define DCF_oe		DDR_B4
#define DCF			PORT_B4

#define LED			PORT_B0
#define LED_oe		DDR_B0

#define LED_ON	0   // STK500
#define LED_OFF 1

#define HIGH 1
#define LOW	 0


struct time {
  uint8_t second;
  uint8_t minute;
  uint8_t hour;
  uint8_t day;
  uint8_t wday;
  uint8_t month;
  uint8_t year;
};

struct time disp_time, dcf_time;
volatile uint8_t sekunde_flag;


/*****************************************************/
ISR(TIMER0_OVF_vect)
{
	static uint8_t ticks_8ms;

	if (++ticks_8ms == 125) 
	{
		sekunde_flag = 1;
		ticks_8ms=0;
	}
	TCNT0 = TIMER_SET;
}

/****************************************************/

void shift_out(uint8_t bit)
{
	DCF = HIGH; 
//	LED=LED_ON;
	if (bit&0x01) _delay_ms(200);
	else _delay_ms(100);
	DCF=LOW;
//	LED=LED_OFF;
}

 	uint8_t temp, shift=0, parity;  // wegen Simulation


int main()
{

	uint8_t leap_second; 

	DCF=0; LED= LED_OFF;
	DCF_oe = 1; // output
	LED_oe = OUTPUT;

	TCCR0B = (1<<CS02); // Prescale 256 = 31.25kHz oder 32us
	TCNT0 = TIMER_SET; // 256-250=6 fr Teiler durch 250 bei 8MHz oder =131 bei 4MHz
	TIMSK = (1<<TOIE0); // IRQ enable

//	Time init

	disp_time.minute= 13;
	disp_time.hour = 15;
	disp_time.day = 19;
	disp_time.wday = 4;
	disp_time.month = 5;
	disp_time.year = 17;

	leap_second = 0;

	sei();
	
	while(1)
	{
		switch (disp_time.minute)
		{
			case 15:
			case 20:
			case 25: 
			case 30:
				leap_second = 1;
				break;
			default:
				leap_second = 0; 
		}

		if (sekunde_flag)
		{	
			sekunde_flag=0;
			disp_time.second++;

// Uhr
			if (disp_time.second == 60+leap_second) 
			{   
				disp_time.minute++;
				disp_time.second = 0;

			}
			if (disp_time.minute == 60)
			{
				disp_time.hour++;
				disp_time.minute=0;
			}	
			if (disp_time.hour == 24) 
			{
				disp_time.hour=0;
			}

// Test Schaltsekunden

// DCF serielle Ausgabe, bis 19 nur Nullen, da das eh nicht gewertet wird. 
			if (disp_time.second<19) shift_out(0);
	
			switch (disp_time.second)
			{
				case 19:
					if (leap_second) shift_out(1);  // Ankndigung Schaltsekunde
					else shift_out(0);
					LED=LED_ON;
					break;
				case 20:
					shift_out(1);  // Startbit Zeitinformation
					LED=LED_OFF;
					break;
				case 21:
				case 22:
				case 23:
				case 24:
					shift = disp_time.second - 21;
					temp = disp_time.minute%10;
					temp >>= shift; 
					shift_out (temp & 0x01);
					parity += temp & 0x01;
					break;

				case 25:
				case 26: 
				case 27:
					shift = disp_time.second - 25;
					temp = disp_time.minute/10;
					temp >>= shift; 
					shift_out (temp & 0x01);
					parity += temp & 0x01;
					break;

				case 29:
				case 30:
				case 31:
				case 32:
					shift = disp_time.second - 29;
					temp = disp_time.hour%10;
					temp >>= shift; 
					shift_out (temp & 0x01);
					parity += temp & 0x01;
					break;

				case 33:
				case 34:
					shift = disp_time.second - 33;
					temp = disp_time.hour/10;
					temp >>= shift; 
					shift_out (temp & 0x01);
					parity += temp & 0x01;
					break;

				case 36:
				case 37:
				case 38:
				case 39:
					shift = disp_time.second - 36;
					temp = disp_time.day%10;
					temp >>= shift; 
					shift_out (temp & 0x01);
					parity += temp & 0x01;
					break;

				case 40:
				case 41:
					shift = disp_time.second - 40;
					temp = disp_time.day/10;  // Kalendertag Einer
					temp >>= shift; 
					shift_out (temp & 0x01);
					parity += temp & 0x01;
					break;
				
				case 42: 
				case 43:
				case 44: 
					shift = disp_time.second - 42;
					temp = disp_time.wday;		// Kalendertag Zener
					temp >>= shift; 
					shift_out (temp & 0x01);
					parity += temp & 0x01;
					break;

				case 45:
				case 46:
				case 47:
				case 48:
					shift = disp_time.second - 45;
					temp = disp_time.month%10;  // Monat Einer
					temp >>= shift; 
					shift_out (temp & 0x01);
					parity += temp & 0x01;
					break;

				case 49:
					temp = disp_time.month/10;  // Monat Zehner
					shift_out (temp & 0x01);
					parity += temp & 0x01;
					break;
					
				case 50:
				case 51:
				case 52:
				case 53:
					shift = disp_time.second - 50;
					temp = disp_time.year%10;  // Jahr Einer
					temp >>= shift; 
					shift_out (temp & 0x01);
					parity += temp & 0x01;
					break;

				case 54:
				case 55:
				case 56:
				case 57:
					shift = disp_time.second - 54;
					temp = disp_time.year/10;  // Jahr Zehner
					temp >>= shift; 
					shift_out (temp & 0x01);
					parity += temp & 0x01;
					break;
// Parity bestimmen
				case 28: 
				case 35:
				case 58:
					shift_out(parity&1);
					parity=0;
					break;
				
				case 59:
					if (leap_second) shift_out(0);
					break; 
				default:
					break;  // kein Puls at all
			}



			// hier ev. Schaltsekunde bei Sekunde 59, nomalerweise nichts ....
		
		} // Sekunde Flag 

	}  // Main Loop
}

